@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -28,22 +28,10 @@
 #include <glib-object.h>
 #include "as-component.h"
 
-#define AS_TYPE_DATA_POOL		(as_data_pool_get_type())
-#define AS_DATA_POOL(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_DATA_POOL, AsDataPool))
-#define AS_DATA_POOL_CLASS(cls)		(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_DATA_POOL, AsDataPoolClass))
-#define AS_IS_DATA_POOL(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_DATA_POOL))
-#define AS_IS_DATA_POOL_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_DATA_POOL))
-#define AS_DATA_POOL_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), AS_TYPE_DATA_POOL, AsDataPoolClass))
-
 G_BEGIN_DECLS
 
-typedef struct _AsDataPool	AsDataPool;
-typedef struct _AsDataPoolClass	AsDataPoolClass;
-
-struct _AsDataPool
-{
-	GObject parent;
-};
+#define AS_TYPE_DATA_POOL (as_data_pool_get_type ())
+G_DECLARE_DERIVABLE_TYPE (AsDataPool, as_data_pool, AS, DATA_POOL, GObject)
 
 struct _AsDataPoolClass
 {
@@ -55,26 +43,38 @@ struct _AsDataPoolClass
 	void (*_as_reserved4) (void);
 	void (*_as_reserved5) (void);
 	void (*_as_reserved6) (void);
-	void (*_as_reserved7) (void);
-	void (*_as_reserved8) (void);
 };
 
-GType		 	as_data_pool_get_type (void);
+/**
+ * AsDataPoolError:
+ * @AS_DATA_POOL_ERROR_FAILED:		Generic failure
+ *
+ * A metadata pool error.
+ **/
+typedef enum {
+	AS_DATA_POOL_ERROR_FAILED,
+	/*< private >*/
+	AS_DATA_POOL_ERROR_LAST
+} AsDataPoolError;
+
+#define AS_DATA_POOL_ERROR	as_data_pool_error_quark ()
+GQuark			as_data_pool_error_quark (void);
+
 AsDataPool		*as_data_pool_new (void);
 
-gchar			**as_data_pool_get_watched_locations (AsDataPool *dpool);
-
-gboolean		as_data_pool_update (AsDataPool *dpool);
-GList			*as_data_pool_get_components (AsDataPool *dpool);
-AsComponent		*as_data_pool_get_component_by_id (AsDataPool *dpool,
-								const gchar *id);
-
 const gchar 		*as_data_pool_get_locale (AsDataPool *dpool);
 void			as_data_pool_set_locale (AsDataPool *dpool,
 							const gchar *locale);
 
-void			as_data_pool_set_data_source_directories (AsDataPool *dpool,
-									gchar **dirs);
+gboolean		as_data_pool_update (AsDataPool *dpool, GError **error);
+
+GList			*as_data_pool_get_components (AsDataPool *dpool);
+AsComponent		*as_data_pool_get_component_by_id (AsDataPool *dpool,
+								const gchar *id);
+
+GPtrArray		*as_data_pool_get_metadata_locations (AsDataPool *dpool);
+void			as_data_pool_set_metadata_locations (AsDataPool *dpool,
+								gchar **dirs);
 
 G_END_DECLS
 
