@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -31,22 +31,10 @@
 
 #include "as-component.h"
 
-#define AS_TYPE_METADATA		(as_metadata_get_type())
-#define AS_METADATA(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_METADATA, AsMetadata))
-#define AS_METADATA_CLASS(cls)		(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_METADATA, AsMetadataClass))
-#define AS_IS_METADATA(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_METADATA))
-#define AS_IS_METADATA_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_METADATA))
-#define AS_METADATA_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), AS_TYPE_METADATA, AsMetadataClass))
-
 G_BEGIN_DECLS
 
-typedef struct _AsMetadata	AsMetadata;
-typedef struct _AsMetadataClass	AsMetadataClass;
-
-struct _AsMetadata
-{
-	GObject			parent;
-};
+#define AS_TYPE_METADATA (as_metadata_get_type ())
+G_DECLARE_DERIVABLE_TYPE (AsMetadata, as_metadata, AS, METADATA, GObject)
 
 struct _AsMetadataClass
 {
@@ -58,31 +46,54 @@ struct _AsMetadataClass
 	void (*_as_reserved4)	(void);
 	void (*_as_reserved5)	(void);
 	void (*_as_reserved6)	(void);
-	void (*_as_reserved7)	(void);
-	void (*_as_reserved8)	(void);
 };
 
 /**
- * AsMetadataError:
- * @AS_METADATA_ERROR_FAILED: Generic failure
+ * AsParserMode:
+ * @AS_PARSER_MODE_UPSTREAM:	Parse Appstream upstream metadata
+ * @AS_PARSER_MODE_DISTRO:	Parse Appstream distribution metadata
  *
- * The error type.
+ * There are a few differences between Appstream's upstream metadata
+ * and the distribution metadata.
+ * The parser mode indicates which style we should process.
+ * Usually you don't want to change this.
+ **/
+typedef enum {
+	AS_PARSER_MODE_UPSTREAM,
+	AS_PARSER_MODE_DISTRO,
+	/*< private >*/
+	AS_PARSER_MODE_LAST
+} AsParserMode;
+
+/**
+ * AsMetadataError:
+ * @AS_METADATA_ERROR_FAILED:			Generic failure
+ * @AS_METADATA_ERROR_UNEXPECTED_FORMAT_KIND:	Expected upstream metadata but got distro metadata, or vice versa.
+ * @AS_METADATA_ERROR_NO_COMPONENT:		We expected a component in the pool, but couldn't find one.
+ *
+ * A metadata processing error.
  **/
 typedef enum {
 	AS_METADATA_ERROR_FAILED,
+	AS_METADATA_ERROR_UNEXPECTED_FORMAT_KIND,
+	AS_METADATA_ERROR_NO_COMPONENT,
+	/*< private >*/
 	AS_METADATA_ERROR_LAST
 } AsMetadataError;
 
-#define	AS_METADATA_ERROR		as_metadata_error_quark ()
+#define	AS_METADATA_ERROR	as_metadata_error_quark ()
 
-GType		 	as_metadata_get_type (void);
-AsMetadata*		as_metadata_new (void);
-GQuark			as_metadata_error_quark	(void);
+AsMetadata		*as_metadata_new (void);
+GQuark			as_metadata_error_quark (void);
 
 void			as_metadata_parse_file (AsMetadata *metad,
 							GFile *file,
 							GError **error);
-void			as_metadata_parse_data (AsMetadata *metad,
+
+void			as_metadata_parse_xml (AsMetadata *metad,
+							const gchar *data,
+							GError **error);
+void			as_metadata_parse_yaml (AsMetadata *metad,
 							const gchar *data,
 							GError **error);
 
@@ -90,19 +101,23 @@ AsComponent		*as_metadata_get_component (AsMetadata *metad);
 GPtrArray		*as_metadata_get_components (AsMetadata *metad);
 
 void			as_metadata_clear_components (AsMetadata *metad);
-
 void			as_metadata_add_component (AsMetadata *metad,
 							AsComponent *cpt);
-gchar			*as_metadata_component_to_upstream_xml (AsMetadata *metad);
-gchar			*as_metadata_components_to_distro_xml (AsMetadata *metad);
 
+gchar			*as_metadata_component_to_upstream_xml (AsMetadata *metad);
 void			as_metadata_save_upstream_xml (AsMetadata *metad,
 							const gchar *fname,
 							GError **error);
+gchar			*as_metadata_components_to_distro_xml (AsMetadata *metad);
 void			as_metadata_save_distro_xml (AsMetadata *metad,
 							const gchar *fname,
 							GError **error);
 
+gchar			*as_metadata_components_to_distro_yaml (AsMetadata *metad);
+void			as_metadata_save_distro_yaml (AsMetadata *metad,
+							const gchar *fname,
+							GError **error);
+
 void			as_metadata_set_locale (AsMetadata *metad,
 							const gchar *locale);
 const gchar		*as_metadata_get_locale (AsMetadata *metad);
@@ -111,6 +126,21 @@ const gchar		*as_metadata_get_origin (AsMetadata *metad);
 void			as_metadata_set_origin (AsMetadata *metad,
 							const gchar *origin);
 
+gboolean		as_metadata_get_update_existing (AsMetadata *metad);
+void			as_metadata_set_update_existing (AsMetadata *metad,
+								gboolean update);
+
+gboolean		as_metadata_get_write_header (AsMetadata *metad);
+void			as_metadata_set_write_header (AsMetadata *metad,
+								gboolean wheader);
+
+void			as_metadata_set_parser_mode (AsMetadata *metad,
+							AsParserMode mode);
+AsParserMode		as_metadata_get_parser_mode (AsMetadata *metad);
+
+const gchar		*as_metadata_get_architecture (AsMetadata *metad);
+void			as_metadata_set_architecture (AsMetadata *metad,
+							const gchar *arch);
 
 G_END_DECLS
 
