@@ -30,15 +30,17 @@
  */
 
 #include "as-screenshot.h"
+#include "as-screenshot-private.h"
 
-typedef struct _AsScreenshotPrivate	AsScreenshotPrivate;
-struct _AsScreenshotPrivate
+#include "as-utils-private.h"
+
+typedef struct
 {
-	AsScreenshotKind	kind;
-	GHashTable			*caption;
-	GPtrArray			*images;
-	gchar				*active_locale;
-};
+	AsScreenshotKind kind;
+	GHashTable *caption;
+	GPtrArray *images;
+	gchar *active_locale;
+} AsScreenshotPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsScreenshot, as_screenshot, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (as_screenshot_get_instance_private (o))
@@ -68,7 +70,7 @@ as_screenshot_init (AsScreenshot *screenshot)
 	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
 
 	priv->active_locale = g_strdup ("C");
-	priv->kind = AS_SCREENSHOT_KIND_NORMAL;
+	priv->kind = AS_SCREENSHOT_KIND_EXTRA;
 	priv->images = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	priv->caption = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 }
@@ -96,10 +98,10 @@ as_screenshot_class_init (AsScreenshotClass *klass)
 AsScreenshotKind
 as_screenshot_kind_from_string (const gchar *kind)
 {
-	if (g_strcmp0 (kind, "normal") == 0)
-		return AS_SCREENSHOT_KIND_NORMAL;
 	if (g_strcmp0 (kind, "default") == 0)
 		return AS_SCREENSHOT_KIND_DEFAULT;
+	if (g_strcmp0 (kind, "extra") == 0)
+		return AS_SCREENSHOT_KIND_EXTRA;
 	return AS_SCREENSHOT_KIND_UNKNOWN;
 }
 
@@ -115,10 +117,10 @@ as_screenshot_kind_from_string (const gchar *kind)
 const gchar *
 as_screenshot_kind_to_string (AsScreenshotKind kind)
 {
-	if (kind == AS_SCREENSHOT_KIND_NORMAL)
-		return "normal";
 	if (kind == AS_SCREENSHOT_KIND_DEFAULT)
 		return "default";
+	if (kind == AS_SCREENSHOT_KIND_EXTRA)
+		return "extra";
 	return NULL;
 }
 
@@ -225,7 +227,7 @@ as_screenshot_set_caption (AsScreenshot *screenshot, const gchar *caption, const
 		locale = priv->active_locale;
 
 	g_hash_table_insert (priv->caption,
-				g_strdup (locale),
+				as_locale_strip_encoding (g_strdup (locale)),
 				g_strdup (caption));
 }
 
@@ -276,6 +278,18 @@ as_screenshot_set_active_locale (AsScreenshot *screenshot, const gchar *locale)
 	priv->active_locale = g_strdup (locale);
 }
 
+/**
+ * as_screenshot_get_caption_table:
+ *
+ * Internal function.
+ */
+GHashTable*
+as_screenshot_get_caption_table (AsScreenshot *screenshot)
+{
+	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
+	return priv->caption;
+}
+
 /**
  * as_screenshot_new:
  *
