@@ -30,22 +30,10 @@
 
 #include "as-image.h"
 
-#define AS_TYPE_SCREENSHOT		(as_screenshot_get_type())
-#define AS_SCREENSHOT(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_SCREENSHOT, AsScreenshot))
-#define AS_SCREENSHOT_CLASS(cls)	(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_SCREENSHOT, AsScreenshotClass))
-#define AS_IS_SCREENSHOT(obj)	(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_SCREENSHOT))
-#define AS_IS_SCREENSHOT_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_SCREENSHOT))
-#define AS_SCREENSHOT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), AS_TYPE_SCREENSHOT, AsScreenshotClass))
-
 G_BEGIN_DECLS
 
-typedef struct _AsScreenshot		AsScreenshot;
-typedef struct _AsScreenshotClass	AsScreenshotClass;
-
-struct _AsScreenshot
-{
-	GObject			parent;
-};
+#define AS_TYPE_SCREENSHOT (as_screenshot_get_type ())
+G_DECLARE_DERIVABLE_TYPE (AsScreenshot, as_screenshot, AS, SCREENSHOT, GObject)
 
 struct _AsScreenshotClass
 {
@@ -57,22 +45,20 @@ struct _AsScreenshotClass
 	void (*_as_reserved4)	(void);
 	void (*_as_reserved5)	(void);
 	void (*_as_reserved6)	(void);
-	void (*_as_reserved7)	(void);
-	void (*_as_reserved8)	(void);
 };
 
 /**
  * AsScreenshotKind:
  * @AS_SCREENSHOT_KIND_UNKNOWN:		Type invalid or not known
- * @AS_SCREENSHOT_KIND_NORMAL:		Optional screenshot
- * @AS_SCREENSHOT_KIND_DEFAULT:		Screenshot to show by default
+ * @AS_SCREENSHOT_KIND_DEFAULT:		The primary screenshot to show by default
+ * @AS_SCREENSHOT_KIND_EXTRA:		Optional screenshot
  *
  * The screenshot type.
  **/
 typedef enum {
 	AS_SCREENSHOT_KIND_UNKNOWN,
-	AS_SCREENSHOT_KIND_NORMAL,
 	AS_SCREENSHOT_KIND_DEFAULT,
+	AS_SCREENSHOT_KIND_EXTRA,
 	/*< private >*/
 	AS_SCREENSHOT_KIND_LAST
 } AsScreenshotKind;
@@ -81,7 +67,6 @@ AsScreenshotKind		as_screenshot_kind_from_string (const gchar *kind);
 const gchar			*as_screenshot_kind_to_string (AsScreenshotKind kind);
 gboolean			as_screenshot_is_valid (AsScreenshot *screenshot);
 
-GType		 		as_screenshot_get_type (void);
 AsScreenshot			*as_screenshot_new (void);
 
 AsScreenshotKind		as_screenshot_get_kind (AsScreenshot *screenshot);
