@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
  * Copyright (C)      2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
@@ -31,6 +31,11 @@
 #include <sys/types.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
+#include <time.h>
+#include <utime.h>
+#include <sys/utsname.h>
+#include <sys/stat.h>
+#include <errno.h>
 
 #include "as-category.h"
 
@@ -62,6 +67,9 @@ as_description_markup_convert_simple (const gchar *markup)
 	gchar *formatted = NULL;
 	GString *str = NULL;
 
+	if (markup == NULL)
+		return NULL;
+
 	/* is this actually markup */
 	if (g_strrstr (markup, "<") == NULL) {
 		formatted = g_strdup (markup);
@@ -101,6 +109,8 @@ as_description_markup_convert_simple (const gchar *markup)
 		} else if ((g_strcmp0 ((gchar*) iter->name, "ul") == 0) || (g_strcmp0 ((gchar*) iter->name, "ol") == 0)) {
 			/* iterate over itemize contents */
 			for (iter2 = iter->children; iter2 != NULL; iter2 = iter2->next) {
+				if (iter2->type != XML_ELEMENT_NODE)
+					continue;
 				if (g_strcmp0 ((gchar*) iter2->name, "li") == 0) {
 					content = (gchar*) xmlNodeGetContent (iter2);
 					g_string_append_printf (str,
@@ -133,6 +143,12 @@ out:
 	return formatted;
 }
 
+/**
+ * as_str_empty:
+ * @str: The string to test.
+ *
+ * Test if a C string is NULL or empty (containing only spaces).
+ */
 gboolean
 as_str_empty (const gchar* str)
 {
@@ -141,34 +157,46 @@ as_str_empty (const gchar* str)
 	return FALSE;
 }
 
-gboolean
-as_utils_touch_dir (const gchar* dirname)
+/**
+ * as_iso8601_to_datetime:
+ *
+ * Helper function to work around a bug in g_time_val_from_iso8601.
+ * Can be dropped when the bug gets resolved upstream:
+ * https://bugzilla.gnome.org/show_bug.cgi?id=760983
+ **/
+GDateTime*
+as_iso8601_to_datetime (const gchar *iso_date)
 {
-	GFile *d = NULL;
-	GError *error = NULL;
-	g_return_val_if_fail (dirname != NULL, FALSE);
+	GTimeVal tv;
+	guint dmy[] = {0, 0, 0};
 
-	d = g_file_new_for_path (dirname);
-	if (!g_file_query_exists (d, NULL)) {
-		g_file_make_directory_with_parents (d, NULL, &error);
-		if (error != NULL) {
-			g_critical ("Unable to create directory tree. Error: %s", error->message);
-			g_error_free (error);
-			return FALSE;
-		}
+	/* nothing set */
+	if (iso_date == NULL || iso_date[0] == '\0')
+		return NULL;
+
+	/* try to parse complete ISO8601 date */
+	if (g_strstr_len (iso_date, -1, " ") != NULL) {
+		if (g_time_val_from_iso8601 (iso_date, &tv) && tv.tv_sec != 0)
+			return g_date_time_new_from_timeval_utc (&tv);
 	}
-	g_object_unref (d);
 
-	return TRUE;
+	/* g_time_val_from_iso8601() blows goats and won't
+	 * accept a valid ISO8601 formatted date without a
+	 * time value - try and parse this case */
+	if (sscanf (iso_date, "%u-%u-%u", &dmy[0], &dmy[1], &dmy[2]) != 3)
+		return NULL;
+
+	/* create valid object */
+	return g_date_time_new_utc (dmy[0], dmy[1], dmy[2], 0, 0, 0);
 }
 
 /**
  * as_utils_delete_dir_recursive:
  * @dirname: Directory to remove
  *
- * Remove folder like rm -r does
+ * Remove directory and all its children (like rm -r does).
  *
- * Returns: TRUE if operation was successful
+ * Returns: %TRUE if operation was successful
  */
 gboolean
 as_utils_delete_dir_recursive (const gchar* dirname)
@@ -229,7 +257,7 @@ out:
  *
  * Create a list of categories from string array
  *
- * Returns: (element-type AsCategory) (transfer full): #GPtrArray of #AsCategory objcts matching the strings in the array
+ * Returns: (element-type AsCategory) (transfer full): #GPtrArray of #AsCategory objects matching the strings in the array
  */
 GPtrArray*
 as_utils_categories_from_strv (gchar** categories_strv, GPtrArray* system_categories)
@@ -297,34 +325,42 @@ as_utils_categories_from_str (const gchar* categories_str, GPtrArray* system_cat
 	return cat_list;
 }
 
+/**
+ * as_utils_find_files_matching:
+ */
 GPtrArray*
-as_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean recursive)
+as_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean recursive, GError **error)
 {
 	GPtrArray* list;
-	GError *error = NULL;
 	GFileInfo *file_info;
 	GFileEnumerator *enumerator = NULL;
 	GFile *fdir;
+	GError *tmp_error = NULL;
 	g_return_val_if_fail (dir != NULL, NULL);
 	g_return_val_if_fail (pattern != NULL, NULL);
 
 	list = g_ptr_array_new_with_free_func (g_free);
 	fdir =  g_file_new_for_path (dir);
-	enumerator = g_file_enumerate_children (fdir, G_FILE_ATTRIBUTE_STANDARD_NAME, 0, NULL, &error);
-	if (error != NULL)
+	enumerator = g_file_enumerate_children (fdir, G_FILE_ATTRIBUTE_STANDARD_NAME, 0, NULL, &tmp_error);
+	if (tmp_error != NULL)
 		goto out;
 
-	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &error)) != NULL) {
+	while ((file_info = g_file_enumerator_next_file (enumerator, NULL, &tmp_error)) != NULL) {
 		gchar *path;
+
+		if (tmp_error != NULL)
+			goto out;
 		if (g_file_info_get_is_hidden (file_info))
 			continue;
+
 		path = g_build_filename (dir,
 					 g_file_info_get_name (file_info),
 					 NULL);
+
 		if ((!g_file_test (path, G_FILE_TEST_IS_REGULAR)) && (recursive)) {
 			GPtrArray *subdir_list;
 			guint i;
-			subdir_list = as_utils_find_files_matching (path, pattern, recursive);
+			subdir_list = as_utils_find_files_matching (path, pattern, recursive, &tmp_error);
 			/* if there was an error, exit */
 			if (subdir_list == NULL) {
 				g_ptr_array_unref (list);
@@ -346,15 +382,17 @@ as_utils_find_files_matching (const gchar* dir, const gchar* pattern, gboolean r
 			g_ptr_array_add (list, path);
 		}
 	}
-	if (error != NULL)
-		goto out;
+
 
 out:
 	g_object_unref (fdir);
 	if (enumerator != NULL)
 		g_object_unref (enumerator);
-	if (error != NULL) {
-		fprintf (stderr, "Error while finding files in directory %s: %s\n", dir, error->message);
+	if (tmp_error != NULL) {
+		if (error == NULL)
+			g_debug ("Error while searching for files in %s: %s", dir, tmp_error->message);
+		else
+			g_propagate_error (error, tmp_error);
 		g_ptr_array_unref (list);
 		return NULL;
 	}
@@ -362,17 +400,22 @@ out:
 	return list;
 }
 
+/**
+ * as_utils_find_files:
+ */
 GPtrArray*
-as_utils_find_files (const gchar* dir, gboolean recursive)
+as_utils_find_files (const gchar *dir, gboolean recursive, GError **error)
 {
 	GPtrArray* res = NULL;
 	g_return_val_if_fail (dir != NULL, NULL);
 
-	res = as_utils_find_files_matching (dir, "", recursive);
+	res = as_utils_find_files_matching (dir, "", recursive, error);
 	return res;
 }
 
-
+/**
+ * as_utils_is_root:
+ */
 gboolean
 as_utils_is_root (void)
 {
@@ -381,27 +424,41 @@ as_utils_is_root (void)
 	return (vuid == ((uid_t) 0));
 }
 
-gchar*
-as_string_strip (const gchar* str)
+/**
+ * as_utils_is_writable:
+ * @path: the path to check.
+ *
+ * Checks if a path is writable.
+ */
+gboolean
+as_utils_is_writable (const gchar *path)
 {
-	gchar* result = NULL;
-	gchar* _tmp0_ = NULL;
-	g_return_val_if_fail (str != NULL, NULL);
-	_tmp0_ = g_strdup (str);
-	result = _tmp0_;
-	g_strstrip (result);
-	return result;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GFileInfo) file_info = NULL;
+
+	file = g_file_new_for_path (path);
+	file_info = g_file_query_info (
+		file,
+		G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+		G_FILE_QUERY_INFO_NONE,
+		NULL,
+		NULL);
+
+	if (file_info && g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+		return g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+
+	return FALSE;
 }
 
 /**
- * as_get_locale:
+ * as_get_current_locale:
  *
  * Returns a locale string as used in the AppStream specification.
  *
  * Returns: (transfer full): A locale string, free with g_free()
  */
 gchar*
-as_get_locale (void)
+as_get_current_locale (void)
 {
 	const gchar * const *locale_names;
 	gchar *tmp;
@@ -442,6 +499,9 @@ as_ptr_array_to_strv (GPtrArray *array)
 	return value;
 }
 
+/**
+ * as_str_replace:
+ */
 gchar*
 as_str_replace (const gchar *str, const gchar *old, const gchar *new)
 {
@@ -473,3 +533,164 @@ as_str_replace (const gchar *str, const gchar *old, const gchar *new)
 
 	return ret;
 }
+
+/**
+ * as_touch_location:
+ * @fname: The file or directory to touch.
+ *
+ * Change mtime of a filesystem location.
+ */
+gboolean
+as_touch_location (const gchar *fname)
+{
+	struct stat sb;
+	struct utimbuf new_times;
+
+	if (stat (fname, &sb) < 0)
+		return FALSE;
+
+	new_times.actime = sb.st_atime;
+	new_times.modtime = time (NULL);
+	if (utime (fname, &new_times) < 0)
+		return FALSE;
+
+	return TRUE;
+}
+
+/**
+ * as_reset_umask:
+ *
+ * Reset umask potentially set by the user to a default value,
+ * so we can create files with the correct permissions.
+ */
+void
+as_reset_umask (void)
+{
+	umask (0022);
+}
+
+/**
+ * as_copy_file:
+ *
+ * Copy a file.
+ */
+gboolean
+as_copy_file (const gchar *source, const gchar *destination, GError **error)
+{
+	FILE *fsrc, *fdest;
+	int a;
+
+	fsrc = fopen (source, "rb");
+	if (fsrc == NULL) {
+		g_set_error (error,
+				G_FILE_ERROR,
+				G_FILE_ERROR_FAILED,
+				"Could not copy file: %s", g_strerror (errno));
+		return FALSE;
+	}
+
+	fdest = fopen (destination, "wb");
+	if (fdest == NULL) {
+		g_set_error (error,
+				G_FILE_ERROR,
+				G_FILE_ERROR_FAILED,
+				"Could not copy file: %s", g_strerror (errno));
+		fclose (fsrc);
+		return FALSE;
+	}
+
+	while (TRUE) {
+		a = fgetc (fsrc);
+
+		if (!feof (fsrc))
+			fputc (a, fdest);
+		else
+			break;
+	}
+
+	fclose (fdest);
+	fclose (fsrc);
+	return TRUE;
+}
+
+/**
+ * as_is_cruft_locale:
+ *
+ * Checks whether the given locale is valid or a cruft or dummy
+ * locale.
+ */
+gboolean
+as_is_cruft_locale (const gchar *locale)
+{
+	if (locale == NULL)
+		return FALSE;
+	if (g_strcmp0 (locale, "x-test") == 0)
+		return TRUE;
+	if (g_strcmp0 (locale, "xx") == 0)
+		return TRUE;
+	return FALSE;
+}
+
+/**
+ * as_locale_strip_encoding:
+ *
+ * Remove the encoding from a locale string.
+ * The function modifies the string directly.
+ */
+gchar*
+as_locale_strip_encoding (gchar *locale)
+{
+	gchar *tmp;
+	tmp = g_strstr_len (locale, -1, ".UTF-8");
+	if (tmp != NULL)
+		*tmp = '\0';
+	return locale;
+}
+
+/**
+ * as_get_current_arch:
+ *
+ * Get the current architecture as vendor strings
+ * (e.g. "amd64" instead of "x86_64").
+ *
+ * Returns: (transfer full): The current OS architecture as string
+ */
+gchar*
+as_get_current_arch (void)
+{
+	gchar *arch;
+	struct utsname uts;
+
+	uname (&uts);
+
+	if (g_strcmp0 (uts.machine, "x86_64") == 0) {
+		arch = g_strdup ("amd64");
+	} else if (g_pattern_match_simple ("i?86", uts.machine)) {
+		arch = g_strdup ("ia32");
+	} else if (g_strcmp0 (uts.machine, "aarch64")) {
+		arch = g_strdup ("arm64");
+	} else {
+		arch = g_strdup (uts.machine);
+	}
+
+	return arch;
+}
+
+/**
+ * as_arch_compatible:
+ * @arch1: Architecture 1
+ * @arch2: Architecture 2
+ *
+ * Compares two architectures and returns %TRUE if they are compatible.
+ */
+gboolean
+as_arch_compatible (const gchar *arch1, const gchar *arch2)
+{
+	if (g_strcmp0 (arch1, arch2) == 0)
+		return TRUE;
+	if (g_strcmp0 (arch1, "all") == 0)
+		return TRUE;
+	if (g_strcmp0 (arch2, "all") == 0)
+		return TRUE;
+	return FALSE;
+}
