@@ -27,22 +27,10 @@
 
 #include <glib-object.h>
 
-#define AS_TYPE_VALIDATOR_ISSUE			(as_validator_issue_get_type())
-#define AS_VALIDATOR_ISSUE(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_VALIDATOR_ISSUE, AsValidatorIssue))
-#define AS_VALIDATOR_ISSUE_CLASS(cls)		(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_VALIDATOR_ISSUE, AsValidatorIssueClass))
-#define AS_IS_VALIDATOR_ISSUE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_VALIDATOR_ISSUE))
-#define AS_IS_VALIDATOR_ISSUE_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_VALIDATOR_ISSUE))
-#define AS_VALIDATOR_ISSUE_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), AS_TYPE_VALIDATOR_ISSUE, AsValidatorIssueClass))
-
 G_BEGIN_DECLS
 
-typedef struct _AsValidatorIssue		AsValidatorIssue;
-typedef struct _AsValidatorIssueClass		AsValidatorIssueClass;
-
-struct _AsValidatorIssue
-{
-	GObject			parent;
-};
+#define AS_TYPE_VALIDATOR_ISSUE (as_validator_issue_get_type ())
+G_DECLARE_DERIVABLE_TYPE (AsValidatorIssue, as_validator_issue, AS, VALIDATOR_ISSUE, GObject)
 
 struct _AsValidatorIssueClass
 {
@@ -86,10 +74,12 @@ typedef enum {
  * @AS_ISSUE_KIND_TAG_NOT_ALLOWED:	A tag is not allowed in the current context
  * @AS_ISSUE_KIND_PROPERTY_MISSING:	A required property is missing
  * @AS_ISSUE_KIND_PROPERTY_INVALID:	A property is invalid
+ * @AS_ISSUE_KIND_VALUE_MISSING:	A value is missing
  * @AS_ISSUE_KIND_VALUE_WRONG:		The value of a tag or property is wrong
  * @AS_ISSUE_KIND_VALUE_ISSUE:		There is an issue with a tag or property value (often non-fatal)
  * @AS_ISSUE_KIND_FILE_MISSING:		A required file or other metadata was missing
  * @AS_ISSUE_KIND_WRONG_NAME:		The naming of an entity is wrong
+ * @AS_ISSUE_KIND_READ_ERROR:		Reading of data failed
  *
  * The issue type.
  **/
@@ -103,15 +93,16 @@ typedef enum {
 	AS_ISSUE_KIND_TAG_NOT_ALLOWED,
 	AS_ISSUE_KIND_PROPERTY_MISSING,
 	AS_ISSUE_KIND_PROPERTY_INVALID,
+	AS_ISSUE_KIND_VALUE_MISSING,
 	AS_ISSUE_KIND_VALUE_WRONG,
 	AS_ISSUE_KIND_VALUE_ISSUE,
 	AS_ISSUE_KIND_FILE_MISSING,
 	AS_ISSUE_KIND_WRONG_NAME,
+	AS_ISSUE_KIND_READ_ERROR,
 	/*< private >*/
 	AS_ISSUE_KIND_LAST
 } AsIssueKind;
 
-GType		 	as_validator_issue_get_type (void);
 AsValidatorIssue	*as_validator_issue_new (void);
 
 AsIssueKind		as_validator_issue_get_kind (AsValidatorIssue *issue);
