@@ -39,25 +39,21 @@
 #include "as-validator.h"
 #include "as-validator-issue.h"
 
-#include "as-metadata.h"
-#include "as-metadata-private.h"
-
 #include "as-utils.h"
 #include "as-utils-private.h"
+#include "as-xmldata.h"
 #include "as-component.h"
 #include "as-component-private.h"
 
-typedef struct _AsValidatorPrivate	AsValidatorPrivate;
-struct _AsValidatorPrivate
+typedef struct
 {
-	GHashTable *issues; /* of utf8 string->AsValidatorIssue objects */
+	GHashTable *issues; /* of utf8:AsValidatorIssue */
 
 	AsComponent *current_cpt;
 	gchar *current_fname;
-};
+} AsValidatorPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsValidator, as_validator, G_TYPE_OBJECT)
-
 #define GET_PRIVATE(o) (as_validator_get_instance_private (o))
 
 /**
@@ -101,7 +97,7 @@ as_validator_add_issue (AsValidator *validator, AsIssueImportance importance, As
 	gchar *buffer;
 	gchar *str;
 	gchar *id_str;
-	_cleanup_free_ gchar *fname = NULL;
+	g_autofree gchar *fname = NULL;
 	AsValidatorIssue *issue;
 	AsValidatorPrivate *priv = GET_PRIVATE (validator);
 
@@ -195,6 +191,7 @@ as_validator_clear_current_cpt (AsValidator *validator)
 
 /**
  * as_validator_clear_issues:
+ * @validator: An instance of #AsValidator.
  *
  * Clears the list of issues
  **/
@@ -426,22 +423,22 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 {
 	gchar *cpttype;
 	xmlNode *iter;
-	AsMetadata *metad;
+	AsXMLData *xdt;
 	AsComponent *cpt;
 	gchar *metadata_license = NULL;
-	gboolean provides_found = FALSE;
 	GHashTable *found_tags;
+	const gchar *summary;
 
 	found_tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	/* validate the resulting AsComponent for sanity */
-	metad = as_metadata_new ();
-	as_metadata_set_locale (metad, "C");
-	as_metadata_set_parser_mode (metad, mode);
+	xdt = as_xmldata_new ();
+	as_xmldata_initialize (xdt, "C", NULL, NULL, NULL, 0);
+	as_xmldata_set_parser_mode (xdt, mode);
 
-	cpt = as_metadata_parse_component_node (metad, root, TRUE, NULL);
-	g_object_unref (metad);
-	g_assert (cpt != NULL);
+	cpt = as_component_new ();
+	as_xmldata_parse_component_node (xdt, root, cpt, NULL);
+	g_object_unref (xdt);
 
 	as_validator_set_current_cpt (validator, cpt);
 
@@ -484,7 +481,7 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 					as_validator_add_issue (validator,
 								AS_ISSUE_IMPORTANCE_WARNING,
 								AS_ISSUE_KIND_VALUE_WRONG,
-								"Component id belongs to a desktop-application, but doesn't resemble the .desktop file name: \"%s\"",
+								"Component id belongs to a desktop-application, but does not resemble the .desktop file name: \"%s\"",
 								node_content);
 			}
 		} else if (g_strcmp0 (node_name, "metadata_license") == 0) {
@@ -495,7 +492,7 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 				as_validator_add_issue (validator,
 							AS_ISSUE_IMPORTANCE_PEDANTIC,
 							AS_ISSUE_KIND_TAG_DUPLICATED,
-							"The 'pkgname' tag appears multiple times. You can maybe create a metapackage containing the data in order to get rid of defining multiple package names per component.");
+							"The 'pkgname' tag appears multiple times. You should evaluate creating a metapackage containing the data in order to avoid defining multiple package names per component.");
 			}
 		} else if (g_strcmp0 (node_name, "source_pkgname") == 0) {
 			if (g_hash_table_contains (found_tags, node_name)) {
@@ -544,7 +541,6 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 			as_validator_check_children_quick (validator, iter, "mimetype", cpt);
 		} else if (g_strcmp0 (node_name, "provides") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
-			provides_found = TRUE;
 		} else if (g_strcmp0 (node_name, "screenshots") == 0) {
 			as_validator_check_children_quick (validator, iter, "screenshot", cpt);
 		} else if (g_strcmp0 (node_name, "project_license") == 0) {
@@ -560,9 +556,20 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 		} else if ((g_strcmp0 (node_name, "languages") == 0) && (mode == AS_PARSER_MODE_DISTRO)) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_children_quick (validator, iter, "lang", cpt);
+		} else if ((g_strcmp0 (node_name, "translation") == 0) && (mode == AS_PARSER_MODE_UPSTREAM)) {
+			g_autofree gchar *prop = NULL;
+			AsTranslationKind trkind;
+			prop = as_validator_check_type_property (validator, cpt, iter);
+			trkind = as_translation_kind_from_string (prop);
+			if (trkind == AS_TRANSLATION_KIND_UNKNOWN) {
+				as_validator_add_issue (validator,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Unknown type '%s' for <translation/> tag.", prop);
+			}
 		} else if (g_strcmp0 (node_name, "extends") == 0) {
 		} else if (g_strcmp0 (node_name, "bundle") == 0) {
-			gchar *prop;
+			g_autofree gchar *prop = NULL;
 			prop = as_validator_check_type_property (validator, cpt, iter);
 			if ((g_strcmp0 (prop, "limba") != 0) && (g_strcmp0 (prop, "xdg-app") != 0)) {
 				as_validator_add_issue (validator,
@@ -570,7 +577,6 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 							AS_ISSUE_KIND_VALUE_WRONG,
 							"Unknown type '%s' for <bundle/> tag.", prop);
 			}
-			g_free (prop);
 		} else if (g_strcmp0 (node_name, "update_contact") == 0) {
 			if (mode == AS_PARSER_MODE_DISTRO) {
 				as_validator_add_issue (validator,
@@ -590,7 +596,7 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 			as_validator_add_issue (validator,
 						AS_ISSUE_IMPORTANCE_WARNING,
 						AS_ISSUE_KIND_TAG_UNKNOWN,
-						"Found invalid tag: '%s'. Non-standard tags have to be prefixed with \"x-\".",
+						"Found invalid tag: '%s'. Non-standard tags must be prefixed with \"x-\".",
 				node_name);
 			tag_valid = FALSE;
 		}
@@ -620,13 +626,49 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 		g_free (metadata_license);
 	}
 
-	if ((!provides_found) && (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP)) {
+	/* check if the summary is sane */
+	summary = as_component_get_summary (cpt);
+	if ((summary != NULL) && ((strstr (summary, "\n") != NULL) || (strstr (summary, "\t") != NULL))) {
 		as_validator_add_issue (validator,
-					AS_ISSUE_IMPORTANCE_PEDANTIC,
-					AS_ISSUE_KIND_TAG_MISSING,
-					"Component describes a desktop-application, but has no 'provides' tag. It should at least define a 'binary' as public interface.");
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"The summary tag must not contain tabs or linebreaks.");
 	}
 
+	/* check if we have a description */
+	if (as_str_empty (as_component_get_description (cpt))) {
+		AsComponentKind cpt_kind;
+		cpt_kind = as_component_get_kind (cpt);
+
+		if ((cpt_kind == AS_COMPONENT_KIND_DESKTOP_APP) ||
+			(cpt_kind == AS_COMPONENT_KIND_FONT)) {
+			as_validator_add_issue (validator,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"The component is missing a long description. Components of this type must have a long description.");
+		} else {
+			as_validator_add_issue (validator,
+					AS_ISSUE_IMPORTANCE_INFO,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"The component is missing a long description. It is recommended to add one.");
+		}
+	}
+
+	/* validate font specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_FONT) {
+		if (!g_str_has_suffix (as_component_get_id (cpt), ".font"))
+			as_validator_add_issue (validator,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"Components of type 'font' must have an AppStream ID with a '.font' suffix.");
+		if (as_component_get_provided_for_kind (cpt, AS_PROVIDED_KIND_FONT) == NULL)
+			as_validator_add_issue (validator,
+					AS_ISSUE_IMPORTANCE_WARNING,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"Type 'font' component, but no font information was provided via a provides/font tag.");
+	}
+
+	/* validate addon specific stuff */
 	if (as_component_get_extends (cpt)->len > 0) {
 		if (as_component_get_kind (cpt) != AS_COMPONENT_KIND_ADDON)
 			as_validator_add_issue (validator,
@@ -647,6 +689,8 @@ as_validator_validate_component_node (AsValidator *validator, xmlNode *root, AsP
 
 /**
  * as_validator_validate_file:
+ * @validator: An instance of #AsValidator.
+ * @metadata_file: An AppStream XML file.
  *
  * Validate an AppStream XML file
  **/
@@ -658,7 +702,7 @@ as_validator_validate_file (AsValidator *validator, GFile *metadata_file)
 	GFileInputStream* fistream;
 	GFileInfo *info = NULL;
 	const gchar *content_type = NULL;
-	_cleanup_free_ gchar *fname = NULL;
+	g_autofree gchar *fname = NULL;
 
 	info = g_file_query_info (metadata_file,
 				G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
@@ -760,6 +804,8 @@ out:
 
 /**
  * as_validator_validate_data:
+ * @validator: An instance of #AsValidator.
+ * @metadata: XML metadata.
  *
  * Validate AppStream XML data
  **/
@@ -827,6 +873,7 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 struct MInfoCheckData {
 	AsValidator *validator;
 	GHashTable *desktop_fnames;
+	gchar *apps_dir;
 };
 
 /**
@@ -837,7 +884,7 @@ struct MInfoCheckData {
 static gboolean
 as_matches_metainfo (const gchar *fname, const gchar *basename)
 {
-	_cleanup_free_ gchar *tmp = NULL;
+	g_autofree gchar *tmp = NULL;
 
 	tmp = g_strdup_printf ("%s.metainfo.xml", basename);
 	if (g_strcmp0 (fname, tmp) == 0)
@@ -867,7 +914,6 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 	as_validator_set_current_cpt (data->validator, cpt);
 	as_validator_set_current_fname (data->validator, fname);
 
-
 	/* check if the fname and the component-id match */
 	tmp = g_strndup (as_component_get_id (cpt),
 				g_strrstr (as_component_get_id (cpt), ".") - as_component_get_id (cpt));
@@ -884,7 +930,46 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 
 	/* check if the referenced .desktop file exists */
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
-		if (!g_hash_table_contains (data->desktop_fnames, as_component_get_id (cpt))) {
+		if (g_hash_table_contains (data->desktop_fnames, as_component_get_id (cpt))) {
+			g_autofree gchar *desktop_fname_full = NULL;
+			g_autoptr(GKeyFile) dfile = NULL;
+			GError *tmp_error = NULL;
+
+			desktop_fname_full = g_build_filename (data->apps_dir, as_component_get_id (cpt), NULL);
+			dfile = g_key_file_new ();
+
+			g_key_file_load_from_file (dfile, desktop_fname_full, G_KEY_FILE_NONE, &tmp_error);
+			if (tmp_error != NULL) {
+				as_validator_add_issue (data->validator,
+						AS_ISSUE_IMPORTANCE_WARNING,
+						AS_ISSUE_KIND_READ_ERROR,
+						"Unable to read associated .desktop file: %s", tmp_error->message);
+				g_error_free (tmp_error);
+				tmp_error = NULL;
+			} else {
+				/* we successfully opened the .desktop file, now perform some checks */
+
+				/* name */
+				if ((g_strcmp0 (as_component_get_name (cpt), "") == 0) &&
+				    (!g_key_file_has_key (dfile, "Desktop Entry", "Name", NULL))) {
+					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
+					as_validator_add_issue (data->validator,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_MISSING,
+							"The component is missing a name (none found in its metainfo or .desktop file)");
+				}
+
+				/* summary */
+				if ((g_strcmp0 (as_component_get_summary (cpt), "") == 0) &&
+				    (!g_key_file_has_key (dfile, "Desktop Entry", "Comment", NULL))) {
+					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
+					as_validator_add_issue (data->validator,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_MISSING,
+							"The component is missing a summary (none found in its metainfo or .desktop file)");
+				}
+			}
+		} else {
 			as_validator_add_issue (data->validator,
 					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_FILE_MISSING,
@@ -898,14 +983,16 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 
 /**
  * as_validator_validate_tree:
+ * @validator: An instance of #AsValidator.
+ * @root_dir: The root directory of the filesystem tree that should be validated.
  *
  * Validate a full directory tree for issues in AppStream metadata.
  **/
 gboolean
 as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 {
-	_cleanup_free_ gchar *metainfo_dir = NULL;
-	_cleanup_free_ gchar *apps_dir = NULL;
+	g_autofree gchar *metainfo_dir = NULL;
+	g_autofree gchar *apps_dir = NULL;
 	GPtrArray *mfiles = NULL;
 	GPtrArray *dfiles = NULL;
 	GHashTable *dfilenames = NULL;
@@ -917,7 +1004,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 	/* cleanup */
 	as_validator_clear_issues (validator);
 
-	metainfo_dir = g_build_filename (root_dir, "usr", "share", "appdata", NULL);
+	metainfo_dir = g_build_filename (root_dir, "usr", "share", "metainfo", NULL);
 	apps_dir = g_build_filename (root_dir, "usr", "share", "applications", NULL);
 
 	/* check if we actually have a directory which could hold metadata */
@@ -944,7 +1031,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 						g_object_unref);
 
 	/* validate all metainfo files */
-	mfiles = as_utils_find_files_matching (metainfo_dir, "*.xml", FALSE);
+	mfiles = as_utils_find_files_matching (metainfo_dir, "*.xml", FALSE, NULL);
 	for (i = 0; i < mfiles->len; i++) {
 		const gchar *fname;
 		GFile *file;
@@ -954,7 +1041,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 		GFileInputStream *fistream;
 		xmlNode* root;
 		xmlDoc *doc;
-		_cleanup_free_ gchar *fname_basename = NULL;
+		g_autofree gchar *fname_basename = NULL;
 
 		fname = (const gchar*) g_ptr_array_index (mfiles, i);
 		file = g_file_new_for_path (fname);
@@ -1024,7 +1111,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 						g_str_equal,
 						g_free,
 						NULL);
-	dfiles = as_utils_find_files_matching (apps_dir, "*.desktop", FALSE);
+	dfiles = as_utils_find_files_matching (apps_dir, "*.desktop", FALSE, NULL);
 	if (dfiles != NULL) {
 		for (i = 0; i < dfiles->len; i++) {
 			const gchar *fname;
@@ -1037,6 +1124,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 	/* validate the component-id <-> filename relations and availability of other metadata */
 	ht_helper.validator = validator;
 	ht_helper.desktop_fnames = dfilenames;
+	ht_helper.apps_dir = apps_dir;
 	g_hash_table_foreach (validated_cpts,
 				(GHFunc) as_validator_analyze_component_metainfo_relation_cb,
 				&ht_helper);
@@ -1056,6 +1144,7 @@ out:
 
 /**
  * as_validator_get_issues:
+ * @validator: An instance of #AsValidator.
  *
  * Get a list of found metadata format issues.
  *
