@@ -28,23 +28,22 @@
 #include <vector>
 #include <glib/gstdio.h>
 
-#include "database-common.hpp"
+#include "database-schema.hpp"
 #include "../as-menu-parser.h"
 #include "../as-component-private.h"
-#include "../as-provides.h"
 
 using namespace std;
-using namespace Appstream;
+using namespace ASCache;
 
 DatabaseRead::DatabaseRead () :
-    m_xapianDB(0)
+    m_dbPath("")
 {
 
 }
 
 DatabaseRead::~DatabaseRead ()
 {
-
+	m_xapianDB.close ();
 }
 
 bool
@@ -63,12 +62,22 @@ DatabaseRead::open (const gchar *dbPath)
 	if (m_dbLocale.empty ())
 		m_dbLocale = "C";
 
-	m_schemaVersion = m_xapianDB.get_metadata ("db-schema-version");
+	try {
+		m_schemaVersion = stoi (m_xapianDB.get_metadata ("db-schema-version"));
+	} catch (...) {
+		g_warning ("Unable to read database schema version, assuming 0.");
+		m_schemaVersion = 0;
+	}
+
+	if (m_schemaVersion != AS_DB_SCHEMA_VERSION) {
+		g_warning ("Attempted to open an old version of the AppStream cache. Please refresh the cache and try again!");
+		return false;
+	}
 
 	return true;
 }
 
-string
+int
 DatabaseRead::getSchemaVersion ()
 {
 	return m_schemaVersion;
@@ -99,131 +108,249 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 
 	// Component name
 	string cptName = doc.get_value (XapianValues::CPTNAME);
-	as_component_set_name (cpt, cptName.c_str (), NULL);
+	if (!cptName.empty ())
+		as_component_set_name (cpt, cptName.c_str (), NULL);
 	cptName = doc.get_value (XapianValues::CPTNAME_UNTRANSLATED);
-	as_component_set_name (cpt, cptName.c_str (), "C");
+	if (!cptName.empty ())
+		as_component_set_name (cpt, cptName.c_str (), "C");
 
 	// Package name
-	string pkgNamesStr = doc.get_value (XapianValues::PKGNAME);
-	gchar **pkgs = g_strsplit (pkgNamesStr.c_str (), "\n", -1);
-	as_component_set_pkgnames (cpt, pkgs);
-	g_strfreev (pkgs);
+	string pkgNamesStr = doc.get_value (XapianValues::PKGNAMES);
+	if (!pkgNamesStr.empty ()) {
+		g_auto(GStrv) pkgs = g_strsplit (pkgNamesStr.c_str (), ";", -1);
+		as_component_set_pkgnames (cpt, pkgs);
+	}
 
 	// Source package name
 	string cptSPkg = doc.get_value (XapianValues::SOURCE_PKGNAME);
-	as_component_set_source_pkgname (cpt, cptSPkg.c_str ());
+	if (!cptSPkg.empty ())
+		as_component_set_source_pkgname (cpt, cptSPkg.c_str ());
 
 	// Origin
 	string cptOrigin = doc.get_value (XapianValues::ORIGIN);
-	as_component_set_origin (cpt, cptOrigin.c_str ());
-
-	// URLs
-	str = doc.get_value (XapianValues::URLS);
-	gchar **urls = g_strsplit (str.c_str (), "\n", -1);
-	for (uint i = 0; urls[i] != NULL; i += 2) {
-		/* urls are stored in form of "type \n url" (so we just need one stringsplit here...) */
-		if (urls[i+1] == NULL)
-			break;
-		AsUrlKind ukind = as_url_kind_from_string (urls[i]);
-		if (ukind != AS_URL_KIND_UNKNOWN)
-			as_component_add_url (cpt, ukind, urls[i+1]);
-	}
-	g_strfreev (urls);
+	if (!cptOrigin.empty ())
+		as_component_set_origin (cpt, cptOrigin.c_str ());
 
 	// Bundles
+	Bundles bundles;
 	str = doc.get_value (XapianValues::BUNDLES);
-	gchar **bundle_ids = g_strsplit (str.c_str (), "\n", -1);
-	for (uint i = 0; bundle_ids[i] != NULL; i += 2) {
-		/* bundle-ids are stored in form of "type \n id" (so we just need one stringsplit here...) */
-		if (bundle_ids[i+1] == NULL)
-			break;
-		AsBundleKind bkind = as_bundle_kind_from_string (bundle_ids[i]);
+	bundles.ParseFromString (str);
+	for (int i = 0; i < bundles.bundle_size (); i++) {
+		const Bundles_Bundle& bdl = bundles.bundle (i);
+		AsBundleKind bkind = (AsBundleKind) bdl.type ();
 		if (bkind != AS_BUNDLE_KIND_UNKNOWN)
-			as_component_add_bundle_id (cpt, bkind, bundle_ids[i+1]);
+			as_component_add_bundle_id (cpt, bkind, bdl.id ().c_str ());
 	}
-	g_strfreev (bundle_ids);
 
-	// Stock icon
-	string appIcon = doc.get_value (XapianValues::ICON);
-	as_component_add_icon (cpt, AS_ICON_KIND_STOCK, 0, 0, appIcon.c_str ());
-
-	// Icon urls
-	str = doc.get_value (XapianValues::ICON_URLS);
-	gchar **icon_urls = g_strsplit (str.c_str (), "\n", -1);
-	GHashTable *cpt_icon_urls = as_component_get_icon_urls (cpt);
-	for (uint i = 0; icon_urls[i] != NULL; i += 2) {
-		/* icon-urls are stored in form of "size \n url"
-		 * A "size" is a string in the form of "WIDTHxHEIGHT", e.g. "64x64"
-		 */
-		if (icon_urls[i+1] == NULL)
-			break;
-		g_hash_table_insert (cpt_icon_urls, g_strdup (icon_urls[i]), g_strdup (icon_urls[i+1]));
+	// Extends
+	string extendsStr = doc.get_value (XapianValues::EXTENDS);
+	if (!extendsStr.empty ()) {
+		g_auto(GStrv) extends = g_strsplit (extendsStr.c_str (), ";", -1);
+		for (uint i = 0; extends[i] != NULL; i++) {
+			as_component_add_extends (cpt, extends[i]);
+		}
+	}
+
+	// Extensions
+	string extsStr = doc.get_value (XapianValues::EXTENSIONS);
+	if (!extsStr.empty ()) {
+		g_auto(GStrv) exts = g_strsplit (extsStr.c_str (), ";", -1);
+		for (uint i = 0; exts[i] != NULL; i++) {
+			as_component_add_extension (cpt, exts[i]);
+		}
+	}
+
+	// URLs
+	Urls urls;
+	str = doc.get_value (XapianValues::URLS);
+	urls.ParseFromString (str);
+	for (int i = 0; i < urls.url_size (); i++) {
+		const Urls_Url& url = urls.url (i);
+		AsUrlKind ukind = (AsUrlKind) url.type ();
+		if (ukind != AS_URL_KIND_UNKNOWN)
+			as_component_add_url (cpt, ukind, url.url ().c_str ());
+	}
+
+	// Icons
+	Icons icons;
+	str = doc.get_value (XapianValues::ICONS);
+	icons.ParseFromString (str);
+	for (int i = 0; i < icons.icon_size (); i++) {
+		const Icons_Icon& pbIcon = icons.icon (i);
+
+		AsIcon *icon = as_icon_new ();
+		as_icon_set_width (icon, pbIcon.width ());
+		as_icon_set_height (icon, pbIcon.height ());
+
+		if (pbIcon.type () == Icons_IconType_REMOTE) {
+			as_icon_set_kind (icon, AS_ICON_KIND_REMOTE);
+			as_icon_set_url (icon, pbIcon.url ().c_str ());
+		} else {
+			as_icon_set_kind (icon, AS_ICON_KIND_CACHED);
+			as_icon_set_filename (icon, pbIcon.url ().c_str ());
+		}
+		as_component_add_icon (cpt, icon);
+		g_object_unref (icon);
 	}
-	g_strfreev (icon_urls);
 
 	// Summary
-	string appSummary = doc.get_value (XapianValues::SUMMARY);
-	as_component_set_summary (cpt, appSummary.c_str (), NULL);
+	string cptSummary = doc.get_value (XapianValues::SUMMARY);
+	if (!cptSummary.empty ())
+		as_component_set_summary (cpt, cptSummary.c_str (), NULL);
 
 	// Long description
-	string appDescription = doc.get_value (XapianValues::DESCRIPTION);
-	as_component_set_description (cpt, appDescription.c_str (), NULL);
+	string cptDescription = doc.get_value (XapianValues::DESCRIPTION);
+	if (!cptDescription.empty ())
+		as_component_set_description (cpt, cptDescription.c_str (), NULL);
 
 	// Categories
 	string categories_str = doc.get_value (XapianValues::CATEGORIES);
-	as_component_set_categories_from_str (cpt, categories_str.c_str ());
+	if (!categories_str.empty ())
+		as_component_set_categories_from_str (cpt, categories_str.c_str ());
 
 	// Provided items
-	string provided_items_str = doc.get_value (XapianValues::PROVIDED_ITEMS);
-	if (!provided_items_str.empty ()) {
-		gchar **pitems_strv = g_strsplit (provided_items_str.c_str (), "\n", -1);
-		GPtrArray *pitems = as_component_get_provided_items (cpt);
-		for (uint i = 0; pitems_strv[i] != NULL; i++) {
-			g_ptr_array_add (pitems,
-						g_strdup (pitems_strv[i]));
+	ASCache::ProvidedItems pbPI;
+	str = doc.get_value (XapianValues::PROVIDED_ITEMS);
+	pbPI.ParseFromString (str);
+	for (int i = 0; i < pbPI.provided_size (); i++) {
+		const ProvidedItems_Provided& pbProv = pbPI.provided (i);
+
+		AsProvided *prov = as_provided_new ();
+		as_provided_set_kind (prov, (AsProvidedKind) pbProv.type ());
+
+		for (int j = 0; j < pbProv.item_size (); j++) {
+			const string& item = pbProv.item (j);
+			as_provided_add_item (prov, item.c_str ());
 		}
-		g_strfreev (pitems_strv);
+
+		as_component_add_provided (cpt, prov);
 	}
 
 	// Screenshot data
-	string screenshot_xml = doc.get_value (XapianValues::SCREENSHOT_DATA);
-	as_component_load_screenshots_from_internal_xml (cpt, screenshot_xml.c_str ());
+	Screenshots screenshots;
+	str = doc.get_value (XapianValues::SCREENSHOTS);
+	screenshots.ParseFromString (str);
+	for (int i = 0; i < screenshots.screenshot_size (); i++) {
+		const Screenshots_Screenshot& pb_scr = screenshots.screenshot (i);
+		AsScreenshot *scr = as_screenshot_new ();
+		as_screenshot_set_active_locale (scr, m_dbLocale.c_str ());
+
+		if (pb_scr.primary ())
+			as_screenshot_set_kind (scr, AS_SCREENSHOT_KIND_DEFAULT);
+		else
+			as_screenshot_set_kind (scr, AS_SCREENSHOT_KIND_EXTRA);
+
+		if (pb_scr.has_caption ())
+			as_screenshot_set_caption (scr, pb_scr.caption ().c_str (), NULL);
+
+		for (int j = 0; j < pb_scr.image_size (); j++) {
+			const Screenshots_Image& pb_img = pb_scr.image (j);
+			AsImage *img = as_image_new ();
+
+			if (pb_img.source ()) {
+				as_image_set_kind (img, AS_IMAGE_KIND_SOURCE);
+			} else {
+				as_image_set_kind (img, AS_IMAGE_KIND_THUMBNAIL);
+			}
+
+			as_image_set_width (img, pb_img.width ());
+			as_image_set_height (img, pb_img.height ());
+			as_image_set_url (img, pb_img.url ().c_str ());
+
+			as_screenshot_add_image (scr, img);
+			g_object_unref (img);
+		}
+
+		as_component_add_screenshot (cpt, scr);
+		g_object_unref (scr);
+	}
 
 	// Compulsory-for-desktop information
 	string compulsory_str = doc.get_value (XapianValues::COMPULSORY_FOR);
-	gchar **strv = g_strsplit (compulsory_str.c_str (), ";", -1);
-	as_component_set_compulsory_for_desktops (cpt, strv);
-	g_strfreev (strv);
+	if (!compulsory_str.empty ()) {
+		g_auto(GStrv) strv = NULL;
+		strv = g_strsplit (compulsory_str.c_str (), ";", -1);
+		as_component_set_compulsory_for_desktops (cpt, strv);
+	}
 
 	// License
 	string license = doc.get_value (XapianValues::LICENSE);
-	as_component_set_project_license (cpt, license.c_str ());
+	if (!license.empty ())
+		as_component_set_project_license (cpt, license.c_str ());
 
 	// Project group
-	string project_group = doc.get_value (XapianValues::PROJECT_GROUP);
-	as_component_set_project_group (cpt, project_group.c_str ());
+	string projectGroup = doc.get_value (XapianValues::PROJECT_GROUP);
+	if (!projectGroup.empty ())
+		as_component_set_project_group (cpt, projectGroup.c_str ());
 
 	// Source package name
 	string developerName = doc.get_value (XapianValues::DEVELOPER_NAME);
-	as_component_set_developer_name (cpt, developerName.c_str (), NULL);
+	if (!developerName.empty ())
+		as_component_set_developer_name (cpt, developerName.c_str (), NULL);
 
 	// Releases data
-	string releases_xml = doc.get_value (XapianValues::RELEASES_DATA);
-	as_component_load_releases_from_internal_xml (cpt, releases_xml.c_str ());
+	Releases pb_rels;
+	str = doc.get_value (XapianValues::RELEASES);
+	pb_rels.ParseFromString (str);
+	for (int i = 0; i < pb_rels.release_size (); i++) {
+		const Releases_Release& pb_rel = pb_rels.release (i);
+		AsRelease *rel = as_release_new ();
+		as_release_set_active_locale (rel, m_dbLocale.c_str ());
 
-	// Languages
-	str = doc.get_value (XapianValues::LANGUAGES);
-	gchar **lang_data = g_strsplit (str.c_str (), "\n", -1);
-	for (uint i = 0; lang_data[i] != NULL; i += 2) {
-		gint64 percentage;
-		/* language information is stored in form of "locale \n <percentage:int>" (so we just need one stringsplit) */
-		if (lang_data[i+1] == NULL)
-			break;
-		percentage = g_ascii_strtoll (lang_data[i+1], NULL, 10);
-		as_component_add_language (cpt, lang_data[i], percentage);
+		as_release_set_version (rel, pb_rel.version ().c_str ());
+		as_release_set_timestamp (rel, pb_rel.unix_timestamp ());
+		if (pb_rel.has_urgency ())
+			as_release_set_urgency (rel, (AsUrgencyKind) pb_rel.urgency ());
+
+		if (pb_rel.has_description ())
+			as_release_set_description (rel, pb_rel.description ().c_str (), NULL);
+
+		// load locations
+		for (int j = 0; j < pb_rel.location_size (); j++)
+			as_release_add_location (rel, pb_rel.location (j).c_str ());
+
+		// load checksums
+		for (int j = 0; j < pb_rel.checksum_size (); j++) {
+			const Releases_Checksum& pb_cs = pb_rel.checksum (j);
+			AsChecksumKind cskind = (AsChecksumKind) pb_cs.type ();
+
+			if (cskind >= AS_CHECKSUM_KIND_LAST) {
+				g_warning ("Found invalid release-checksum type in database for component '%s'", id_str.c_str ());
+				continue;
+			}
+			as_release_set_checksum (rel, pb_cs.value ().c_str (), cskind);
+		}
+
+		// load sizes
+		for (int j = 0; j < pb_rel.size_size (); j++) {
+			const Releases_Size& pb_s = pb_rel.size (j);
+			AsSizeKind skind = (AsSizeKind) pb_s.type ();
+
+			if (skind >= AS_SIZE_KIND_LAST) {
+				g_warning ("Found invalid release-size type in database for component '%s'", id_str.c_str ());
+				continue;
+			}
+			as_release_set_size (rel, pb_s.value (), skind);
+		}
+
+		as_component_add_release (cpt, rel);
+		g_object_unref (rel);
 	}
 
-	// TODO: Read out keywords?
+	// Languages
+	Languages langs;
+	str = doc.get_value (XapianValues::LANGUAGES);
+	langs.ParseFromString (str);
+	for (int i = 0; i < langs.language_size (); i++) {
+		const Languages_Language& lang = langs.language (i);
+
+		as_component_add_language (cpt,
+					   lang.locale ().c_str (),
+					   lang.percentage ());
+	}
+
+	// TODO: Read out keywords? - actually not necessary, since they're already in the database and used by the search engine
+	//       or do we really need to know what the defined keywords have been for each component?
 
 	return cpt;
 }
@@ -247,14 +374,14 @@ Xapian::QueryParser
 DatabaseRead::newAppStreamParser ()
 {
 	Xapian::QueryParser xapian_parser = Xapian::QueryParser ();
-        xapian_parser.set_database (m_xapianDB);
-        xapian_parser.add_boolean_prefix ("pkg", "XP");
-        xapian_parser.add_boolean_prefix ("pkg", "AP");
-        xapian_parser.add_boolean_prefix ("id", "AI");
-        xapian_parser.add_boolean_prefix ("section", "XS");
-        xapian_parser.add_prefix ("pkg_wildcard", "XP");
-        xapian_parser.add_prefix ("pkg_wildcard", "AP");
-        xapian_parser.set_default_op (Xapian::Query::OP_AND);
+	xapian_parser.set_database (m_xapianDB);
+	xapian_parser.add_boolean_prefix ("id", "AI");
+	xapian_parser.add_boolean_prefix ("pkg", "AP");
+	xapian_parser.add_boolean_prefix ("provides", "AE");
+	xapian_parser.add_boolean_prefix ("section", "XS");
+	xapian_parser.add_prefix ("pkg_wildcard", "XP");
+	xapian_parser.add_prefix ("pkg_wildcard", "AP");
+	xapian_parser.set_default_op (Xapian::Query::OP_AND);
         return xapian_parser;
 }
 
@@ -306,28 +433,30 @@ DatabaseRead::getQueryForCategory (gchar *cat_id)
  * search to the given category
  */
 vector<Xapian::Query>
-DatabaseRead::queryListFromSearchEntry (AsSearchQuery *asQuery)
+DatabaseRead::queryListForTermCats (const gchar *term, gchar **categories)
 {
 	// prepare search-term
-	as_search_query_sanitize_search_term (asQuery);
-	string search_term = as_search_query_get_search_term (asQuery);
-	bool searchAll = as_search_query_get_search_all_categories (asQuery);
+	bool globalSearch = false;
+	string search_term = term == NULL ? "" : term;
+	if (categories == NULL)
+		globalSearch = true;
 
-	// generate category query
+	// generate category query (if we are not searching globally in all categories)
 	Xapian::Query category_query = Xapian::Query ();
-	gchar **categories = as_search_query_get_categories (asQuery);
-	string categories_str = "";
-	for (uint i = 0; categories[i] != NULL; i++) {
-		gchar *cat_id = categories[i];
+	if (not globalSearch) {
+		for (uint i = 0; categories[i] != NULL; i++) {
+			gchar *cat_id = categories[i];
 
-		category_query = Xapian::Query (Xapian::Query::OP_OR,
+			category_query = Xapian::Query (Xapian::Query::OP_OR,
 						 category_query,
 						 getQueryForCategory (cat_id));
+		}
 	}
 
 	// empty query returns a query that matches nothing (for performance
 	// reasons)
-	if ((search_term.compare ("") == 0) && (searchAll)) {
+	// We caught the "both criteria zero" case way earlier, so this is just additional safety.
+	if ((search_term.compare ("") == 0) && (globalSearch)) {
 		Xapian::Query vv[2] = { Xapian::Query(), Xapian::Query () };
 		vector<Xapian::Query> res(&vv[0], &vv[0]+2);
 		return res;
@@ -336,8 +465,8 @@ DatabaseRead::queryListFromSearchEntry (AsSearchQuery *asQuery)
 	// we cheat and return a match-all query for single letter searches
 	if (search_term.length () < 2) {
 		Xapian::Query allQuery = addCategoryToQuery (Xapian::Query (""), category_query);
-		// I want C++11!
-        Xapian::Query vv[2] = { allQuery, allQuery };
+		// NOTE: I want C++11!
+		Xapian::Query vv[2] = { allQuery, allQuery };
 		vector<Xapian::Query> res(&vv[0], &vv[0]+2);
 		return res;
 	}
@@ -392,14 +521,13 @@ DatabaseRead::appendSearchResults (Xapian::Enquire enquire, GPtrArray *cptArray)
 }
 
 GPtrArray*
-DatabaseRead::findComponents (AsSearchQuery *asQuery)
+DatabaseRead::findComponents (const gchar *term, gchar **cats)
 {
 	// Create new array to store the AsComponent objects
 	GPtrArray *cptArray = g_ptr_array_new_with_free_func (g_object_unref);
-	vector<Xapian::Query> qlist;
 
 	// "normal" query
-	qlist = queryListFromSearchEntry (asQuery);
+	auto qlist = queryListForTermCats (term, cats);
 	Xapian::Query query = qlist[0];
 	query.serialise ();
 
@@ -466,18 +594,20 @@ DatabaseRead::getComponentById (const gchar *idname)
 }
 
 GPtrArray*
-DatabaseRead::getComponentsByProvides (AsProvidesKind kind, const gchar *value, const gchar *data)
+DatabaseRead::getComponentsByProvides (AsProvidedKind kind, const gchar *item)
 {
 	/* Create new array to store the AsComponent objects */
 	GPtrArray *cptArray = g_ptr_array_new_with_free_func (g_object_unref);
 
 	Xapian::Query item_query;
-	gchar *provides_item;
-	provides_item = as_provides_item_create (kind, value, data);
+	gchar *element_id;
+	element_id = g_strdup_printf ("%s;%s",
+					 as_provided_kind_to_string (kind),
+					 item);
 	item_query = Xapian::Query (Xapian::Query::OP_OR,
-					   Xapian::Query("AE" + string(provides_item)),
+					   Xapian::Query("AE" + string(element_id)),
 					   Xapian::Query ());
-	g_free (provides_item);
+	g_free (element_id);
 
 	item_query.serialise ();
 
@@ -489,24 +619,18 @@ DatabaseRead::getComponentsByProvides (AsProvidesKind kind, const gchar *value,
 }
 
 GPtrArray*
-DatabaseRead::getComponentsByKind (AsComponentKind kinds)
+DatabaseRead::getComponentsByKind (AsComponentKind kind)
 {
 	GPtrArray *cptArray = g_ptr_array_new_with_free_func (g_object_unref);
 
-	Xapian::Query item_query = Xapian::Query();
+	auto kind_query = Xapian::Query (Xapian::Query::OP_OR,
+			   Xapian::Query("AT" + string(as_component_kind_to_string (kind))),
+			   Xapian::Query());
 
-	for (guint i = 0; i < AS_COMPONENT_KIND_LAST;i++) {
-		if ((kinds & (1 << i)) > 0) {
-			item_query = Xapian::Query (Xapian::Query::OP_OR,
-					   Xapian::Query("AT" + string(as_component_kind_to_string ((AsComponentKind) (1 << i)))),
-					   item_query);
-		}
-	}
-
-	item_query.serialise ();
+	kind_query.serialise ();
 
 	Xapian::Enquire enquire = Xapian::Enquire (m_xapianDB);
-	enquire.set_query (item_query);
+	enquire.set_query (kind_query);
 	appendSearchResults (enquire, cptArray);
 
 	return cptArray;
