@@ -75,6 +75,7 @@ test_component ()
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_DESKTOP_APP);
 
+	as_component_set_id (cpt, "org.example.test.desktop");
 	as_component_set_name (cpt, "Test", NULL);
 	as_component_set_summary (cpt, "It does things", NULL);
 
@@ -89,8 +90,22 @@ test_component ()
 	g_object_unref (metad);
 	g_debug ("%s", str2);
 
-	g_assert_cmpstr (str, ==, "<?xml version=\"1.0\"?>\n<component type=\"desktop\"><name>Test</name><summary>It does things</summary><pkgname>fedex</pkgname></component>\n");
-	g_assert_cmpstr (str2, ==, "<?xml version=\"1.0\"?>\n<components version=\"0.8\"><component type=\"desktop\"><name>Test</name><summary>It does things</summary><pkgname>fedex</pkgname></component></components>\n");
+	g_assert_cmpstr (str, ==, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+				  "<component type=\"desktop\">\n"
+				  "  <id>org.example.test.desktop</id>\n"
+				  "  <name>Test</name>\n"
+				  "  <summary>It does things</summary>\n"
+				  "  <pkgname>fedex</pkgname>\n"
+				  "</component>\n");
+	g_assert_cmpstr (str2, ==, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+				   "<components version=\"0.8\">\n"
+				   "  <component type=\"desktop\">\n"
+				   "    <id>org.example.test.desktop</id>\n"
+				   "    <name>Test</name>\n"
+				   "    <summary>It does things</summary>\n"
+				   "    <pkgname>fedex</pkgname>\n"
+				   "  </component>\n"
+				   "</components>\n");
 
 	g_free (str);
 	g_free (str2);
