@@ -25,8 +25,10 @@
 #include <locale.h>
 
 #include "ascli-utils.h"
-#include "ascli-mdata-actions.h"
-#include "ascli-validate-actions.h"
+#include "ascli-actions-mdata.h"
+#include "ascli-actions-validate.h"
+#include "ascli-actions-pkgmgr.h"
+#include "ascli-actions-misc.h"
 
 static gboolean optn_show_version = FALSE;
 static gboolean optn_verbose_mode = FALSE;
@@ -52,17 +54,23 @@ as_client_get_summary ()
 				/* these are commands we can use with appstream-cli */
 				_("Subcommands:"));
 
-	g_string_append_printf (string, "  %s - %s\n", "search [TERM]", _("Search the component database"));
-	g_string_append_printf (string, "  %s - %s\n", "get [COMPONENT-ID]", _("Get information about a component by its ID"));
-	g_string_append_printf (string, "  %s - %s\n", "what-provides [TYPE] [VALUE]", _("Get components which provide the given item"));
+	g_string_append_printf (string, "  %s - %s\n", "search [TERM]", _("Search the component database."));
+	g_string_append_printf (string, "  %s - %s\n", "get [COMPONENT-ID]", _("Get information about a component by its ID."));
+	g_string_append_printf (string, "  %s - %s\n", "what-provides [TYPE] [VALUE]", _("Get components which provide the given item."));
 	g_string_append_printf (string, "    %s - %s\n", "[TYPE]", _("A provides-item type (e.g. lib, bin, python3, ...)"));
-	g_string_append_printf (string, "    %s - %s\n", "[VALUE]", _("Select a value for the provides-item which needs to be found"));
+	g_string_append_printf (string, "    %s - %s\n", "[VALUE]", _("Select a value for the provides-item which needs to be found."));
 	g_string_append (string, "\n");
-	g_string_append_printf (string, "  %s - %s\n", "dump [COMPONENT-ID]", _("Dump raw XML metadata for a component matching the ID"));
-	g_string_append_printf (string, "  %s - %s\n", "refresh-index", _("Rebuild the component information cache"));
+	g_string_append_printf (string, "  %s - %s\n", "dump [COMPONENT-ID]", _("Dump raw XML metadata for a component matching the ID."));
+	g_string_append_printf (string, "  %s - %s\n", "refresh-index", _("Rebuild the component information cache."));
 	g_string_append (string, "\n");
-	g_string_append_printf (string, "  %s - %s\n", "validate", _("Validate AppStream XML files for issues"));
-	g_string_append_printf (string, "  %s - %s\n", "validate-tree", _("Validate an installed file-tree of an application for valid metadata"));
+	g_string_append_printf (string, "  %s - %s\n", "validate [FILE]", _("Validate AppStream XML files for issues."));
+	g_string_append_printf (string, "  %s - %s\n", "validate-tree [DIRECTORY]", _("Validate an installed file-tree of an application for valid metadata."));
+	g_string_append (string, "\n");
+	g_string_append_printf (string, "  %s - %s\n", "install [COMPONENT-ID]", _("Install software matching the component-id."));
+	g_string_append_printf (string, "  %s - %s\n", "remove  [COMPONENT-ID]", _("Remove software matching the component-id."));
+	g_string_append (string, "\n");
+	g_string_append_printf (string, "  %s - %s\n", "put [FILE]", _("Install a metadata file into the right location."));
+	g_string_append_printf (string, "  %s - %s\n", "status", _("Display various information about installed AppStream metadata."));
 
 	return g_string_free (string, FALSE);
 }
@@ -80,7 +88,6 @@ as_client_run (char **argv, int argc)
 	gchar *command = NULL;
 	gchar *value1 = NULL;
 	gchar *value2 = NULL;
-	gchar *value3 = NULL;
 
 	gchar *summary;
 	gchar *options_help = NULL;
@@ -144,8 +151,6 @@ as_client_run (char **argv, int argc)
 		value1 = argv[2];
 	if (argc > 3)
 		value2 = argv[3];
-	if (argc > 4)
-		value3 = argv[4];
 
 	if ((g_strcmp0 (command, "search") == 0) || (g_strcmp0 (command, "s") == 0)) {
 		exit_code = ascli_search_component (optn_dbpath,
@@ -165,11 +170,19 @@ as_client_run (char **argv, int argc)
 							value1,
 							optn_no_cache);
 	} else if (g_strcmp0 (command, "what-provides") == 0) {
-		exit_code = ascli_what_provides (optn_dbpath, value1, value2, value3, optn_details);
+		exit_code = ascli_what_provides (optn_dbpath, value1, value2, optn_details);
 	} else if (g_strcmp0 (command, "validate") == 0) {
 		exit_code = ascli_validate_files (&argv[2], argc-2, optn_no_color, optn_pedantic);
 	} else if (g_strcmp0 (command, "validate-tree") == 0) {
 		exit_code = ascli_validate_tree (value1, optn_no_color, optn_pedantic);
+	} else if (g_strcmp0 (command, "install") == 0) {
+		exit_code = ascli_install_component (value1);
+	} else if (g_strcmp0 (command, "remove") == 0) {
+		exit_code = ascli_remove_component (value1);
+	} else if (g_strcmp0 (command, "put") == 0) {
+		exit_code = ascli_put_metainfo (value1);
+	} else if (g_strcmp0 (command, "status") == 0) {
+		exit_code = ascli_show_status ();
 	} else {
 		ascli_print_stderr (_("Command '%s' is unknown."), command);
 		exit_code = 1;
