@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <glib/gi18n-lib.h>
+#include "as-utils-private.h"
 
 /**
  * Set to true if we don't want colored output
@@ -34,29 +35,33 @@ gboolean _nocolor_output = FALSE;
 gchar*
 ascli_format_long_output (const gchar *str)
 {
-	gchar *res;
-	gchar *str2;
-	gchar **strv;
-	guint i;
+	GString *res = NULL;
+	guint i, j;
 	gboolean do_linebreak = FALSE;
 
-	str2 = g_strdup (str);
-	for (i = 0; str2[i] != '\0'; ++i) {
-		if ((i != 0) && ((i % 80) == 0))
+	res = g_string_new ("\n  ");
+
+	for (i = 0; str[i] != '\0'; ++i) {
+		if ((j != 0) && ((j % 80) == 0))
 			do_linebreak = TRUE;
-		if ((do_linebreak) && (str2[i] == ' ')) {
+		if (str[i] == '\n') {
 			do_linebreak = FALSE;
-			str2[i] = '\n';
+			g_string_append (res, "\n  ");
+			j = 0;
+			continue;
 		}
+		if ((do_linebreak) && (str[i] == ' ')) {
+			do_linebreak = FALSE;
+			g_string_append (res, "\n  ");
+			j = 0;
+			continue;
+		}
+
+		g_string_append_c (res, str[i]);
+		j++;
 	}
 
-	strv = g_strsplit (str2, "\n", -1);
-	g_free (str2);
-
-	res = g_strjoinv ("\n  ", strv);
-	g_strfreev (strv);
-
-	return res;
+	return g_string_free (res, FALSE);
 }
 
 /**
@@ -93,6 +98,19 @@ ascli_print_key_value (const gchar* key, const gchar* val, gboolean highlight)
 	g_free (fmtval);
 }
 
+/**
+ * ascli_print_highlight:
+ */
+void
+ascli_print_highlight (const gchar* msg)
+{
+	if (_nocolor_output) {
+		g_print ("%s\n", msg);
+	} else {
+		g_print ("%c[%dm%s%c[%dm\n", 0x1B, 1, msg, 0x1B, 0);
+	}
+}
+
 /**
  * ascli_print_separator:
  */
@@ -142,15 +160,6 @@ ascli_print_stdout (const gchar *format, ...)
 	g_free (str);
 }
 
-/**
- * string_hashtable_to_str:
- */
-static void
-string_hashtable_to_str (gchar *key, gchar *value, GString *gstr)
-{
-	g_string_append_printf (gstr, "%s:%s, ", key, value);
-}
-
 /**
  * ascli_set_colored_output:
  */
@@ -166,21 +175,58 @@ ascli_set_colored_output (gboolean colored)
 static gchar*
 as_get_bundle_str (AsComponent *cpt)
 {
-	GHashTable *bundle_ids;
+	guint i;
 	GString *gstr;
 
-	bundle_ids = as_component_get_bundle_ids (cpt);
-	if (g_hash_table_size (bundle_ids) <= 0)
+	if (!as_component_has_bundle (cpt))
 		return NULL;
 
 	gstr = g_string_new ("");
-	g_hash_table_foreach (bundle_ids, (GHFunc) string_hashtable_to_str, gstr);
+	for (i = 0; i < AS_BUNDLE_KIND_LAST; i++) {
+		AsBundleKind kind = (AsBundleKind) i;
+		const gchar *bundle_id;
+
+		bundle_id = as_component_get_bundle_id (cpt, kind);
+		if (bundle_id == NULL)
+			continue;
+		g_string_append_printf (gstr, "%s:%s, ",
+					as_bundle_kind_to_string (kind),
+					bundle_id);
+
+	}
 	if (gstr->len > 0)
 		g_string_truncate (gstr, gstr->len - 2);
 
 	return g_string_free (gstr, FALSE);
 }
 
+/**
+ * ascli_ptrarray_to_pretty:
+ *
+ * Pretty-print a GPtrArray.
+ */
+static gchar*
+ascli_ptrarray_to_pretty (GPtrArray *array)
+{
+	GString *rstr = NULL;
+	guint i;
+
+	if (array->len == 1) {
+		return g_strdup (g_ptr_array_index (array, 0));
+	}
+
+	rstr = g_string_new ("");
+	for (i = 0; i < array->len; i++) {
+		const gchar *astr = (const gchar*) g_ptr_array_index (array, i);
+
+		g_string_append_printf (rstr, "- %s\n", astr);
+	}
+	if (rstr->len > 0)
+		g_string_truncate (rstr, rstr->len - 1);
+
+	return g_string_free (rstr, FALSE);
+}
+
 /**
  * ascli_print_component:
  *
@@ -192,22 +238,25 @@ ascli_print_component (AsComponent *cpt, gboolean show_detailed)
 	gchar *short_idline;
 	gchar *pkgs_str = NULL;
 	gchar *bundles_str = NULL;
+	AsIcon *icon = NULL;
 	guint j;
 
 	short_idline = g_strdup_printf ("%s [%s]",
-							as_component_get_id (cpt),
-							as_component_kind_to_string (as_component_get_kind (cpt)));
+					as_component_get_id (cpt),
+					as_component_kind_to_string (as_component_get_kind (cpt)));
 	if (as_component_get_pkgnames (cpt) != NULL)
 		pkgs_str = g_strjoinv (", ", as_component_get_pkgnames (cpt));
 	bundles_str = as_get_bundle_str (cpt);
 
+	icon = as_component_get_icon_by_size (cpt, 64, 64);
+
 	ascli_print_key_value (_("Identifier"), short_idline, FALSE);
 	ascli_print_key_value (_("Name"), as_component_get_name (cpt), FALSE);
 	ascli_print_key_value (_("Summary"), as_component_get_summary (cpt), FALSE);
 	ascli_print_key_value (_("Package"), pkgs_str, FALSE);
 	ascli_print_key_value (_("Bundle"), bundles_str, FALSE);
 	ascli_print_key_value (_("Homepage"), as_component_get_url (cpt, AS_URL_KIND_HOMEPAGE), FALSE);
-	ascli_print_key_value (_("Icon"), as_component_get_icon_url (cpt, 64, 64), FALSE);
+	ascli_print_key_value (_("Icon"), icon == NULL ? NULL : as_icon_get_name (icon), FALSE);
 	g_free (short_idline);
 	g_free (pkgs_str);
 	g_free (bundles_str);
@@ -216,7 +265,10 @@ ascli_print_component (AsComponent *cpt, gboolean show_detailed)
 	if (show_detailed) {
 		GPtrArray *sshot_array;
 		GPtrArray *imgs = NULL;
-		GPtrArray *provided_items;
+		GPtrArray *extends;
+		GPtrArray *extensions;
+		GList *provided;
+		GList *l;
 		AsScreenshot *sshot;
 		AsImage *img;
 		gchar *str;
@@ -225,6 +277,14 @@ ascli_print_component (AsComponent *cpt, gboolean show_detailed)
 		/* developer name */
 		ascli_print_key_value (_("Developer"), as_component_get_developer_name (cpt), FALSE);
 
+		/* extends data (e.g. for addons) */
+		extends = as_component_get_extends (cpt);
+		if (extends != NULL) {
+			str = ascli_ptrarray_to_pretty (extends);
+			ascli_print_key_value (_("Extends"), str, FALSE);
+			g_free (str);
+		}
+
 		/* long description */
 		str = as_description_markup_convert_simple (as_component_get_description (cpt));
 		ascli_print_key_value (_("Description"), str, FALSE);
@@ -275,14 +335,33 @@ ascli_print_component (AsComponent *cpt, gboolean show_detailed)
 			g_free (str);
 		}
 
-		/* Provided Items */
-		provided_items = as_component_get_provided_items (cpt);
-		strv = as_ptr_array_to_strv (provided_items);
-		if (strv != NULL) {
-			str = g_strjoinv (" ", strv);
-			ascli_print_key_value (_("Provided Items"), str, FALSE);
+		/* list of addons extending this component */
+		extensions = as_component_get_extensions (cpt);
+		if (extensions != NULL) {
+			str = ascli_ptrarray_to_pretty (extensions);
+			ascli_print_key_value (_("Extensions"), str, FALSE);
 			g_free (str);
 		}
-		g_strfreev (strv);
+
+		/* Provided Items */
+		provided = as_component_get_provided (cpt);
+		if (provided != NULL)
+			ascli_print_key_value (_("Provided Items"), "↓", FALSE);
+		for (l = provided; l != NULL; l = l->next) {
+			g_autofree gchar **items = NULL;
+			AsProvided *prov = AS_PROVIDED (l->data);
+
+			items = as_provided_get_items (prov);
+			if (items != NULL) {
+				g_autofree gchar *keyname = NULL;
+
+				str = g_strjoinv (" ", items);
+				keyname = g_strdup_printf (" %s",
+								as_provided_kind_to_l10n_string (as_provided_get_kind (prov)));
+
+				ascli_print_key_value (keyname, str, FALSE);
+				g_free (str);
+			}
+		}
 	}
 }
